==> cds-services-impl-1.18.3-sources.jar: com/sap/cds/services/impl/environment/DefaultEnvFileUtils.java <==
package com.sap.cds.services.impl.environment;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.nio.charset.StandardCharsets;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.Scanner;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.sap.cds.services.runtime.CdsRuntime;
import com.sap.cds.services.utils.StringUtils;

class DefaultEnvFileUtils {

	private final static Logger logger = LoggerFactory.getLogger(DefaultEnvFileUtils.class);
	private final static String defaultEnv = "default-env.json";
	private static final String CLASSPATH = "classpath:";

	/**
	 * @return the value of the default-env.json file as String.
	 */
	static String getDefaultEnv(CdsRuntime runtime) {
		// look at the configured path
		InputStream stream = getFromDefaultEnvPath(runtime.getEnvironment().getCdsProperties().getEnvironment().getLocal().getDefaultEnvPath());

		// try to read default-env.json from classpath resources
		if (stream == null) {
			stream = getClassLoaderResource(defaultEnv);
		}

		// now try looking for default-env.json inside the working directory
		Path workingDirectory = Paths.get(System.getProperty("user.dir"));
		if (stream == null) {
			stream = getFromDirectory(workingDirectory, defaultEnv);
		}

		// try within parent of working directory
		if (stream == null) {
			stream = getFromDirectory(workingDirectory.getParent(), defaultEnv);
		}

		if (stream != null) {
			try (Scanner scanner = new Scanner(stream, StandardCharsets.UTF_8.name())) {
				return scanner.useDelimiter("\\A").next();
			}
		}
		return null;
	}

	private static InputStream getFromDirectory(Path workingDirectory, String fileName) {
		File defaultEnvFile = workingDirectory.resolve(fileName).toFile();
		if (defaultEnvFile.exists()) {
			try {
				logger.info("Loaded {} from directory '{}'", defaultEnvFile.getName(), defaultEnvFile.getParent());
				return new FileInputStream(defaultEnvFile);
			} catch (FileNotFoundException e) { // NOSONAR
				// ignore
			}
		}
		return null;
	}

	private static InputStream getFromDefaultEnvPath(String defaultEnvPath) {
		if (StringUtils.isEmpty(defaultEnvPath)) {
			return null;
		}

		// try to read it as classloader resource
		if (defaultEnvPath.startsWith(CLASSPATH)) {
			return getClassLoaderResource(defaultEnvPath.substring(CLASSPATH.length()));
		}

		// check, if the path exists and is a file or directory
		File defaultEnvFile = new File(defaultEnvPath);
		if (defaultEnvFile.isDirectory()) {
			// configured as directory, use default file name
			defaultEnvFile = Paths.get(defaultEnvPath).resolve(defaultEnv).toFile();
		}
		if (defaultEnvFile.exists()) {
			try {
				logger.info("Loaded {} from directory '{}'", defaultEnvFile.getName(), defaultEnvFile.getParent());
				return new FileInputStream(defaultEnvFile);
			} catch (FileNotFoundException e) { // NOSONAR
				// ignore
			}
		}

		logger.warn("No default env file found in configured path '{}'", defaultEnvPath);
		return null;
	}

	private static InputStream getClassLoaderResource(String resourceName) {
		InputStream stream = Thread.currentThread().getContextClassLoader().getResourceAsStream(resourceName);
		if (stream != null) {
			logger.info("Loaded {} from classpath resources", resourceName);
		}
		return stream;
	}

}

==> cds-services-impl-1.20.1-sources.jar: com/sap/cds/services/impl/environment/DefaultEnvFileUtils.java <==
package com.sap.cds.services.impl.environment;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.nio.charset.StandardCharsets;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.Scanner;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.sap.cds.services.runtime.CdsRuntime;
import com.sap.cds.services.utils.StringUtils;

class DefaultEnvFileUtils {

	private final static Logger logger = LoggerFactory.getLogger(DefaultEnvFileUtils.class);
	private final static String defaultEnv = "default-env.json";
	private static final String CLASSPATH = "classpath:";

	/**
	 * @return the value of the default-env.json file as String.
	 */
	static String getDefaultEnv(CdsRuntime runtime) {
		// look at the configured path
		InputStream stream = getFromDefaultEnvPath(runtime.getEnvironment().getCdsProperties().getEnvironment().getLocal().getDefaultEnvPath());

		// try to read default-env.json from classpath resources
		if (stream == null) {
			stream = getClassLoaderResource(defaultEnv);
		}

		// now try looking for default-env.json inside the working directory
		Path workingDirectory = Paths.get(System.getProperty("user.dir"));

		if (stream == null) {
			stream = getFromDirectory(workingDirectory, defaultEnv);
		}

		// try within parent of working directory, parent could be null in docker container
		if (stream == null && workingDirectory.getParent() != null) {
			stream = getFromDirectory(workingDirectory.getParent(), defaultEnv);
		}

		if (stream != null) {
			try (Scanner scanner = new Scanner(stream, StandardCharsets.UTF_8.name())) {
				return scanner.useDelimiter("\\A").next();
			}
		}
		return null;
	}

	private static InputStream getFromDirectory(Path workingDirectory, String fileName) {
		File defaultEnvFile = workingDirectory.resolve(fileName).toFile();
		if (defaultEnvFile.exists()) {
			try {
				logger.info("Loaded {} from directory '{}'", defaultEnvFile.getName(), defaultEnvFile.getParent());
				return new FileInputStream(defaultEnvFile);
			} catch (FileNotFoundException e) { // NOSONAR
				// ignore
			}
		}
		return null;
	}

	private static InputStream getFromDefaultEnvPath(String defaultEnvPath) {
		if (StringUtils.isEmpty(defaultEnvPath)) {
			return null;
		}

		// try to read it as classloader resource
		if (defaultEnvPath.startsWith(CLASSPATH)) {
			return getClassLoaderResource(defaultEnvPath.substring(CLASSPATH.length()));
		}

		// check, if the path exists and is a file or directory
		File defaultEnvFile = new File(defaultEnvPath);
		if (defaultEnvFile.isDirectory()) {
			// configured as directory, use default file name
			defaultEnvFile = Paths.get(defaultEnvPath).resolve(defaultEnv).toFile();
		}
		if (defaultEnvFile.exists()) {
			try {
				logger.info("Loaded {} from directory '{}'", defaultEnvFile.getName(), defaultEnvFile.getParent());
				return new FileInputStream(defaultEnvFile);
			} catch (FileNotFoundException e) { // NOSONAR
				// ignore
			}
		}

		logger.warn("No default env file found in configured path '{}'", defaultEnvPath);
		return null;
	}

	private static InputStream getClassLoaderResource(String resourceName) {
		InputStream stream = Thread.currentThread().getContextClassLoader().getResourceAsStream(resourceName);
		if (stream != null) {
			logger.info("Loaded {} from classpath resources", resourceName);
		}
		return stream;
	}

}
